@@ -143,7 +143,7 @@ class FibreChannelConnector(base.BaseLinuxConnector):
             for device in host_devices:
                 LOG.debug("Looking for Fibre Channel dev %(device)s",
                           {'device': device})
-                if os.path.exists(device):
+                if os.path.exists(device) and self.check_valid_device(device):
                     self.host_device = device
                     # get the /dev/sdX device.  This is used
                     # to find the multipath device.
@@ -243,7 +243,8 @@ class FibreChannelConnector(base.BaseLinuxConnector):
 
     @utils.trace
     @synchronized('connect_volume')
-    def disconnect_volume(self, connection_properties, device_info):
+    def disconnect_volume(self, connection_properties, device_info,
+                          force=False, ignore_errors=False):
         """Detach the volume from instance_name.
 
         :param connection_properties: The dictionary that describes all
@@ -272,14 +273,21 @@ class FibreChannelConnector(base.BaseLinuxConnector):
             devices.append(device_info)
 
         LOG.debug("devices to remove = %s", devices)
-        self._remove_devices(connection_properties, devices)
+        self._remove_devices(connection_properties, devices, device_info)
 
-    def _remove_devices(self, connection_properties, devices):
+    def _remove_devices(self, connection_properties, devices, device_info):
         # There may have been more than 1 device mounted
         # by the kernel for this volume.  We have to remove
         # all of them
+        path_used = self._linuxscsi.get_dev_path(connection_properties,
+                                                 device_info)
+        was_multipath = '/pci-' not in path_used
         for device in devices:
-            self._linuxscsi.remove_scsi_device(device["device"])
+            device_path = device['device']
+            flush = self._linuxscsi.requires_flush(device_path,
+                                                   path_used,
+                                                   was_multipath)
+            self._linuxscsi.remove_scsi_device(device_path, flush=flush)
 
     def _get_pci_num(self, hba):
         # NOTE(walter-boring)
