@@ -43,6 +43,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
 
     supported_transports = ['be2iscsi', 'bnx2i', 'cxgb3i', 'default',
                             'cxgb4i', 'qla4xxx', 'ocs', 'iser']
+    VALID_SESSIONS_PREFIX = ('tcp:', 'iser:')
 
     def __init__(self, root_helper, driver=None,
                  execute=None, use_multipath=False,
@@ -245,14 +246,13 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         else:
             LOG.info("Multipath discovery for iSCSI not enabled.")
             iscsi_sessions = self._get_iscsi_sessions()
-            iscsi_portals_with_sessions = [s[2] for s in iscsi_sessions]
 
             host_devices = set()
             for props in self._iterate_all_targets(connection_properties):
                 # If we aren't trying to connect to the portal, we
                 # want to find ALL possible paths from all of the
                 # alternate portals
-                if props['target_portal'] in iscsi_portals_with_sessions:
+                if props['target_portal'] in iscsi_sessions:
                     paths = self._get_device_path(props)
                     host_devices.update(paths)
             host_devices = list(host_devices)
@@ -783,7 +783,8 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         nodes = self._get_iscsi_nodes()
         sessions = self._get_iscsi_sessions_full()
         # Use (portal, iqn) to map the session value
-        sessions_map = {(s[2], s[4]): s[1] for s in sessions if s[0] == 'tcp:'}
+        sessions_map = {(s[2], s[4]): s[1] for s in sessions
+                        if s[0] in self.VALID_SESSIONS_PREFIX}
         # device_map will keep a tuple with devices from the connection and
         # others that don't belong to this connection" (belong, others)
         device_map = collections.defaultdict(lambda: (set(), set()))
@@ -835,10 +836,12 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         :type ignore_errors: bool
         """
         return self._cleanup_connection(connection_properties, force=force,
-                                        ignore_errors=ignore_errors)
+                                        ignore_errors=ignore_errors,
+                                        device_info=device_info)
 
     def _cleanup_connection(self, connection_properties, ips_iqns_luns=None,
-                            force=False, ignore_errors=False):
+                            force=False, ignore_errors=False,
+                            device_info=None):
         """Cleans up connection flushing and removing devices and multipath.
 
         :param connection_properties: The dictionary that describes all
@@ -854,6 +857,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         :param ignore_errors: When force is True, this will decide whether to
                               ignore errors or raise an exception once finished
                               the operation.  Default is False.
+        :param device_info: Attached device information.
         :type ignore_errors: bool
         """
         exc = exception.ExceptionChainer()
@@ -870,9 +874,14 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         remove_devices = set()
         for remove, __ in devices_map.values():
             remove_devices.update(remove)
-        multipath_name = self._linuxscsi.remove_connection(remove_devices,
-                                                           self.use_multipath,
-                                                           force, exc)
+
+        path_used = self._linuxscsi.get_dev_path(connection_properties,
+                                                 device_info)
+        was_multipath = (path_used.startswith('/dev/dm-') or
+                         'mpath' in path_used)
+        multipath_name = self._linuxscsi.remove_connection(
+            remove_devices, self.use_multipath, force, exc,
+            path_used, was_multipath)
 
         # Disconnect sessions and remove nodes that are left without devices
         disconnect = [conn for conn, (__, keep) in devices_map.items()
@@ -974,7 +983,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         iqns = []
         for data in [line.split() for line in output.splitlines()]:
             if len(data) == 2 and data[1].startswith('iqn.'):
-                ips.append(data[0])
+                ips.append(data[0].split(',')[0])
                 iqns.append(data[1])
         return ips, iqns
 
@@ -1021,7 +1030,8 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
             sessions = self._get_iscsi_sessions_full()
             for s in sessions:
                 # Found our session, return session_id
-                if 'tcp:' == s[0] and portal == s[2] and s[4] == target_iqn:
+                if (s[0] in self.VALID_SESSIONS_PREFIX and
+                        portal == s[2] and s[4] == target_iqn):
                     return s[1], manual_scan
 
             try:
